import * as path from "path";
import ts from "typescript";
import { parseJson } from "../../../utils";
const knownMissingSettings = [
    "tslint.alwaysShowRuleFailuresAsWarnings",
    "tslint.exclude",
    "tslint.ignoreDefinitionFiles",
    "tslint.jsEnable",
    "tslint.suppressWhileTypeErrorsPresent",
];
export const convertVSCodeConfig = (rawEditorSettings, settings) => {
    var _a;
    const editorSettings = parseJson(rawEditorSettings);
    const missing = knownMissingSettings.filter((setting) => editorSettings[setting]);
    const autoFixOnSave = editorSettings["editor.codeActionsOnSave"] &&
        typeof editorSettings["editor.codeActionsOnSave"] === "object" &&
        ((_a = editorSettings["editor.codeActionsOnSave"]) === null || _a === void 0 ? void 0 : _a["source.fixAll.tslint"]);
    // Only create a new config file path if the input and output configs roughly match
    const eslintPathMatches = editorSettings["tslint.configFile"] &&
        typeof editorSettings["tslint.configFile"] === "string" &&
        !path.relative(path.dirname(editorSettings["tslint.configFile"]), path.dirname(settings.config));
    // We can bail without making changes if there are no changes we need to make
    if (!autoFixOnSave && !eslintPathMatches) {
        return { contents: rawEditorSettings, missing };
    }
    // Since we've found at least one matching setting, we know the source structure is a proper {}
    const sourceFile = ts.createSourceFile("settings.json", rawEditorSettings, ts.ScriptTarget.Latest, 
    /*setParentNodes*/ true, ts.ScriptKind.JSON);
    const jsonRoot = sourceFile.statements[0]
        .expression;
    const propertyIndexByName = (properties, name) => properties.findIndex((property) => property.name && ts.isStringLiteral(property.name) && property.name.text === name);
    const transformer = (context) => (rootNode) => {
        const upsertProperties = (node, additions) => {
            const originalProperties = node.properties;
            for (const [parent, setting, value] of additions) {
                const createNewChild = (properties) => {
                    return context.factory.createPropertyAssignment(`"${parent}"`, context.factory.createObjectLiteralExpression([
                        ...(properties !== null && properties !== void 0 ? properties : []),
                        context.factory.createPropertyAssignment(`"${setting}"`, typeof value === "string"
                            ? context.factory.createStringLiteral(value)
                            : context.factory.createTrue()),
                    ], true));
                };
                const existingIndex = propertyIndexByName(originalProperties, parent);
                if (existingIndex !== -1) {
                    const existingProperty = originalProperties[existingIndex];
                    const updatedProperties = [...node.properties];
                    // We know these casts should be safe because we previously found a matching parent object for the property
                    updatedProperties[existingIndex] = createNewChild(existingProperty
                        .initializer.properties);
                    node = context.factory.createObjectLiteralExpression(updatedProperties, true);
                }
                else {
                    node = context.factory.createObjectLiteralExpression([...node.properties, createNewChild()], true);
                }
            }
            return node;
        };
        const visit = (node) => {
            node = ts.visitEachChild(node, visit, context);
            if (node !== jsonRoot) {
                return node;
            }
            const additions = [];
            if (autoFixOnSave !== undefined) {
                additions.push([
                    "editor.codeActionsOnSave",
                    "eslint.autoFixOnSave",
                    autoFixOnSave,
                ]);
            }
            if (eslintPathMatches !== undefined) {
                additions.push(["eslint.options", "configFile", settings.config]);
            }
            return upsertProperties(jsonRoot, additions);
        };
        return ts.visitNode(rootNode, visit);
    };
    const printer = ts.createPrinter(undefined);
    const result = ts.transform(sourceFile, [transformer]);
    const contents = printer
        .printFile(result.transformed[0])
        .replace(/^\(/giu, "")
        .replace(/\);(\r\n|\r|\n)*$/giu, "$1");
    result.dispose();
    return { contents, missing };
};
//# sourceMappingURL=convertVSCodeConfig.js.map